#ifndef __sockrpc_HXX
#include "sockrpc.hxx"
#endif

#ifndef __dbvue_svc_HXX
#define __dbvue_svc_HXX

//  YOODA : Yet another Object Oriented Database
//    Copyright (C) 1994  Eric Abecassis

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.

// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

#ifdef __GNUG__
#pragma interface
#endif

class RemoteDBVue : public ServerDBVue , public SvcServer {
  public :
  RemoteDBVue(int family=AF_INET) : SvcServer("RemoteDBVue", family) {};
  RemoteDBVue(char* n, int f) : SvcServer(n,f) {}


  private :

  ListVal<PID> _locks ;
  DeltaVue* _deltavue ;
  proxy(CallbackServer) _callback ; 
  int _pid ; // PID of the client 
  char* _usid ; // String which identify the client 

  public :

  RemoteDBVue(void*) ;
  ~RemoteDBVue() ;
  void garbage(const PID& pid) ;	
  void disconnect() ;
  int isConnected() { return this->SvcServer::isConnected() ; }	
  int svcerror() { return this->error() ; }
  void createLog() ;
  char* clientname() { return _usid ; }
  int clientpid() { return _pid ; }
  static void MTTM_exec() ;

  public :

  void connect(char* userid, int pid, char* callb, int delta) ;
  void upgrade( ListVal<PID>& in,  ListVal<PID>& eject) ;
  void release(int delta, ListVal<PID>& in, 
	        ListVal<PID>& k,  ListVal<PID>& nk) ; 
  int sync() ; 
  void abort(int delta) ;
  int secondphase() ;
  void write(const Page& p, const PID& pid) ;
  void freepage(const PID& pid) ;
  void readfirst( Page& p, const PID& pid) ;
  void readnext( Page& p, const PID& pid) ;
  PID allocation(int volNo, int num) ;
  int lock(const PID& pid, const Lock& l) ;

  public :

  class ReqRegister_connect {
    public :
    int dummy ;
    ReqRegister_connect(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_connect,0);
    } 
  } req_connect ;
  static void svc_connect(SvcSocket& __svc) {
    char* userid ;
    int pid ;
    char* callb ;
    int delta ;
    __svc >> userid >> pid >> callb >> delta ;
    ( ((RemoteDBVue*)(&__svc))->connect(userid, pid, callb, delta) );
    delete userid ;
    delete callb ;
  } ;

  class ReqRegister_upgrade {
    public :
    int dummy ;
    ReqRegister_upgrade(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_upgrade,1);
    } 
  } req_upgrade ;
  static void svc_upgrade(SvcSocket& __svc) {
    ListVal<PID> in ;
    ListVal<PID> eject ;
    __svc >> in ;
    ( ((RemoteDBVue*)(&__svc))->upgrade(in, eject) );
    __svc << eject << svcflush ;
  } ;

  class ReqRegister_release {
    public :
    int dummy ;
    ReqRegister_release(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_release,2);
    } 
  } req_release ;
  static void svc_release(SvcSocket& __svc) {
    int delta ;
    ListVal<PID> in ;
    ListVal<PID> k ;
    ListVal<PID> nk ;
    __svc >> delta >> in ;
    ( ((RemoteDBVue*)(&__svc))->release(delta, in, k, nk) );
    __svc << k << nk << svcflush ;
  } ;

  class ReqRegister_sync {
    public :
    int dummy ;
    ReqRegister_sync(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_sync,3);
    } 
  } req_sync ;
  static void svc_sync(SvcSocket& __svc) {
    __svc << ( ((RemoteDBVue*)(&__svc))->sync() ) << svcflush ;
  } ;

  class ReqRegister_abort {
    public :
    int dummy ;
    ReqRegister_abort(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_abort,4);
    } 
  } req_abort ;
  static void svc_abort(SvcSocket& __svc) {
    int delta ;
    __svc >> delta ;
    ( ((RemoteDBVue*)(&__svc))->abort(delta) );
  } ;

  class ReqRegister_secondphase {
    public :
    int dummy ;
    ReqRegister_secondphase(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_secondphase,5);
    } 
  } req_secondphase ;
  static void svc_secondphase(SvcSocket& __svc) {
    __svc << ( ((RemoteDBVue*)(&__svc))->secondphase() ) << svcflush ;
  } ;

  class ReqRegister_write {
    public :
    int dummy ;
    ReqRegister_write(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_write,6);
    } 
  } req_write ;
  static void svc_write(SvcSocket& __svc) {
    Page p ;
    PID pid ;
    __svc >> p >> pid ;
    ( ((RemoteDBVue*)(&__svc))->write(p, pid) );
  } ;

  class ReqRegister_freepage {
    public :
    int dummy ;
    ReqRegister_freepage(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_freepage,7);
    } 
  } req_freepage ;
  static void svc_freepage(SvcSocket& __svc) {
    PID pid ;
    __svc >> pid ;
    ( ((RemoteDBVue*)(&__svc))->freepage(pid) );
  } ;

  class ReqRegister_readfirst {
    public :
    int dummy ;
    ReqRegister_readfirst(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_readfirst,8);
    } 
  } req_readfirst ;
  static void svc_readfirst(SvcSocket& __svc) {
    Page p ;
    PID pid ;
    __svc >> pid ;
    ( ((RemoteDBVue*)(&__svc))->readfirst(p, pid) );
    __svc << p << svcflush ;
  } ;

  class ReqRegister_readnext {
    public :
    int dummy ;
    ReqRegister_readnext(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_readnext,9);
    } 
  } req_readnext ;
  static void svc_readnext(SvcSocket& __svc) {
    Page p ;
    PID pid ;
    __svc >> pid ;
    ( ((RemoteDBVue*)(&__svc))->readnext(p, pid) );
    __svc << p << svcflush ;
  } ;

  class ReqRegister_allocation {
    public :
    int dummy ;
    ReqRegister_allocation(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_allocation,10);
    } 
  } req_allocation ;
  static void svc_allocation(SvcSocket& __svc) {
    int volNo ;
    int num ;
    __svc >> volNo >> num ;
    __svc << ( ((RemoteDBVue*)(&__svc))->allocation(volNo, num) ) << svcflush ;
  } ;

  class ReqRegister_lock {
    public :
    int dummy ;
    ReqRegister_lock(SvcServer* svc = SvcServer::_svcBase) {
      svc->reqRegister(RemoteDBVue::svc_lock,11);
    } 
  } req_lock ;
  static void svc_lock(SvcSocket& __svc) {
    PID pid ;
    Lock l ;
    __svc >> pid >> l ;
    __svc << ( ((RemoteDBVue*)(&__svc))->lock(pid, l) ) << svcflush ;
  } ;
} ;



#endif
